@@ -4,6 +4,8 @@ import collections
 import datetime
 import functools
 import io
+import os
+import random
 import re
 from typing import Any
 from typing import Callable
@@ -43,6 +45,16 @@ NSX_DEFENDER_PORTAL_URLS = {
 }
 NSX_DEFENDER_PORTAL_LB_URL = "https://user.lastline.com"
 
+# Sometimes we want to know the data-center
+URL_TO_DATA_CENTER = {
+    NSX_DEFENDER_ANALYSIS_URLS[NSX_DEFENDER_DC_WESTUS]: NSX_DEFENDER_DC_WESTUS,
+    NSX_DEFENDER_ANALYSIS_URLS[NSX_DEFENDER_DC_NLEMEA]: NSX_DEFENDER_DC_NLEMEA,
+    NSX_DEFENDER_ANALYSIS_LB_URL: NSX_DEFENDER_DC_WESTUS,
+    NSX_DEFENDER_PORTAL_URLS[NSX_DEFENDER_DC_WESTUS]: NSX_DEFENDER_DC_WESTUS,
+    NSX_DEFENDER_PORTAL_URLS[NSX_DEFENDER_DC_NLEMEA]: NSX_DEFENDER_DC_NLEMEA,
+    NSX_DEFENDER_PORTAL_LB_URL: NSX_DEFENDER_DC_WESTUS,
+}
+
 
 # Metadata types that can be returned from 'AnalysisClient.get_result'
 METADATA_TYPE_PCAP = "traffic_capture"
@@ -51,6 +63,8 @@ METADATA_TYPE_YARA_STRINGS = "codehash_yara_strings"
 METADATA_TYPE_CODEHASH = "codehash"
 METADATA_TYPE_SCREENSHOT = "screenshot"
 METADATA_TYPE_SFC = "sfc2_feature_reuse_report"
+METADATA_TYPE_LLBIST_TRACE = "llbist_analyzer_trace"
+METADATA_TYPE_STRINGS = "extracted_strings_report"
 
 # Report types
 REPORT_TYPE_SANDBOX = "ll-int-win"
@@ -61,6 +75,33 @@ DATETIME_MSEC_FMT = DATETIME_FMT + ".%f"
 DATE_FMT = "%Y-%m-%d"
 
 
+# Structures
+EventDescriptor = collections.namedtuple(
+    "EventDescriptor",
+    [
+        "event_id",
+        "event_time",
+        "obfuscated_key_id",
+        "obfuscated_subkey_id",
+        "data_center",
+    ],
+)
+
+
+def is_valid_config_file(file_path: str) -> str:
+    """
+    Validate the path to the configuration file.
+
+    :param str file_path: the path to the config file
+    :rtype: str
+    :return: the validated file path
+    :raises ValueError: if the path is not valid
+    """
+    if not os.path.isfile(file_path):
+        raise ValueError(f"Invalid file path '{file_path}' for configuration file")
+    return file_path
+
+
 def is_likely_task_uuid(hash_like: str) -> bool:
     """
     Heuristic to check whether a string is likely a task uuid.
@@ -202,6 +243,72 @@ def merge_dicts(
     return merged_dict
 
 
+def get_file_paths(
+    dir_name: str,
+    extension: Optional[str] = None,
+    limit: Optional[int] = None,
+    shuffle: bool = False,
+) -> List[str]:
+    """
+    Return all absolute paths inside a directory
+
+    :param str dir_name: the path to the directory
+    :param str|None extension: optional extension filter
+    :param str|None limit: optional limit
+    :param bool shuffle: whether to shuffle the output
+    :rtype: list[str]
+    :return: list of file paths
+    """
+    if not os.path.exists(dir_name):
+        return []
+    if os.path.isfile(dir_name):
+        file_paths = [os.path.abspath(dir_name)]
+    else:
+        file_paths = set(
+            [
+                os.path.abspath(os.path.join(dp, f))
+                for dp, _, filenames in os.walk(dir_name)
+                for f in filenames
+                if f.endswith(extension or "")
+            ]
+        )
+    file_paths = sorted(file_paths)[:limit]
+    if shuffle:
+        # Shuffle deterministically
+        random.Random(4).shuffle(file_paths)
+    return file_paths
+
+
+def parse_portal_link(portal_link: str) -> Optional["EventDescriptor"]:
+    """
+    Parse a portal link.
+
+    :param str portal_link: portal link pointing to the network event
+    :rtype: EventDescriptor|None
+    :return: the EventDescriptor if the parsing succeeded
+    """
+    url_parsed = parse.urlsplit(portal_link, allow_fragments=False)
+
+    try:
+        url_query_string = parse.parse_qs(url_parsed.query)
+        event_time = url_query_string["event_time"][0]
+    except (ValueError, KeyError, IndexError):
+        event_time = None
+
+    try:
+        url_scheme_netloc = parse.urlunsplit((url_parsed.scheme, url_parsed.netloc, "", "", ""))
+        matches = re.search(".*event/(.*)/(.*)/(.*)", url_parsed.path)
+        return EventDescriptor(
+            obfuscated_key_id=matches.group(1),
+            obfuscated_subkey_id=matches.group(2),
+            event_id=matches.group(3),
+            event_time=event_time,
+            data_center=URL_TO_DATA_CENTER[url_scheme_netloc],
+        )
+    except (AttributeError, KeyError):
+        return None
+
+
 class NamedBytesIO(io.BytesIO):
     """Buffer I/O with a name."""
 
