@@ -8,10 +8,12 @@ import datetime
 import hashlib
 import io
 import logging
+import time
 import weakref
 from typing import Any
 from typing import Callable
 from typing import Dict
+from typing import Generator
 from typing import Iterable
 from typing import List
 from typing import Optional
@@ -91,7 +93,7 @@ class AbstractClient(abc.ABC):
         verify_ssl = conf.getboolean(section_name, "verify_ssl", fallback=True)
         return {
             data_center: cls(
-                api_url=cls.HOSTED_URLS[data_center],
+                api_url=data_center_url,
                 login_params=login_params,
                 timeout=timeout,
                 verify_ssl=verify_ssl,
@@ -364,7 +366,7 @@ class AbstractClient(abc.ABC):
         params: Optional[Dict[str, str]] = None,
         fmt: Optional[str] = "json",
         raw: bool = False,
-    ) -> Union[dict, bytes, tau_clients.NamedBytesIO]:
+    ) -> Union[list, dict, bytes, tau_clients.NamedBytesIO]:
         """Utility method to issue a GET request (see '_request' for documentation)."""
         return self._request(
             method="GET",
@@ -379,11 +381,16 @@ class AbstractClient(abc.ABC):
 class PortalClient(AbstractClient):
     """Simple client to query the user portal API (PAPI)."""
 
-    MODULES = ("analysis", "knowledgebase", "login")
+    MODULES = (
+        "analysis",
+        "knowledgebase",
+        "login",
+        "net",
+    )
 
     FORMATS = ("json",)
 
-    HOSTED_URLS = tau_clients.NSX_DEFENDER_PORTAL_URLS
+    HOSTED_URLS = {k: f"{v}/papi" for k, v in tau_clients.NSX_DEFENDER_PORTAL_URLS.items()}
 
     def _login(self) -> None:
         """Implement interface (portal client relies on 'username' and 'password')."""
@@ -391,6 +398,88 @@ class PortalClient(AbstractClient):
             self._session = requests.sessions.session()
         self.post("login", function=None, data=self._login_params)
 
+    def get_event(
+        self,
+        event_id: str,
+        obfuscated_key_id: str,
+        obfuscated_subkey_id: str,
+    ) -> Optional[Dict]:
+        """
+        Get event information.
+
+        :param str event_id: the id of the event
+        :param str obfuscated_key_id: the obfuscated key id
+        :param str obfuscated_subkey_id: the obfuscated sensor key
+        :rtype: dict|None
+        :return: the information, or None if no event is found
+        """
+        params = tau_clients.purge_none(
+            {
+                "event_id": event_id,
+                "key_id": obfuscated_key_id,
+                "subkey_id": obfuscated_subkey_id,
+            }
+        )
+        try:
+            ret = self.get("net", "event/get", params=params)
+            return ret[0]
+        except IndexError:
+            return None
+
+    def get_event_info(
+        self,
+        event_id: str,
+        event_time: str,
+        obfuscated_key_id: str,
+        obfuscated_subkey_id: str,
+    ) -> List[Dict[str, Any]]:
+        """
+        Get PCAP info about an event.
+
+        :param str event_id: the id of the event
+        :param str event_time: when the even took place
+        :param str obfuscated_key_id: the obfuscated key id
+        :param str obfuscated_subkey_id: the obfuscated sensor key
+        :rtype: list[dict[str, any]]
+        :return: get PCAP event information
+        """
+        params = tau_clients.purge_none(
+            {
+                "event_id": event_id,
+                "event_time": event_time,
+                "key_id": obfuscated_key_id,
+                "subkey_id": obfuscated_subkey_id,
+            }
+        )
+        return self.get("net", "pcap/event_info", params=params)
+
+    def get_pcap(
+        self,
+        pcap_id: str,
+        event_time: str,
+        obfuscated_key_id: str,
+        obfuscated_subkey_id: str,
+    ) -> "tau_clients.NamedBytesIO":
+        """
+        Get PCAP.
+
+        :param str pcap_id: the PCAP id
+        :param str event_time: when the even took place
+        :param str obfuscated_key_id: the obfuscated key id
+        :param str obfuscated_subkey_id: the obfuscated sensor key
+        :rtype: NamedBytesIO
+        :return: the PCAP data
+        """
+        params = tau_clients.purge_none(
+            {
+                "pcap_id": pcap_id,
+                "event_time": event_time,
+                "key_id": obfuscated_key_id,
+                "subkey_id": obfuscated_subkey_id,
+            }
+        )
+        return self.get("net", "pcap/get_pcap", params=params, raw=True, fmt=None)
+
     def get_tasks_from_knowledgebase(self, query_string: str, **kwargs) -> List[Dict[str, Any]]:
         """
         Query tasks from the knowledgebase, most recent first.
@@ -466,7 +555,7 @@ class PortalClient(AbstractClient):
         bypass_cache: bool = False,
     ) -> Dict[str, Any]:
         """
-        Upload an URL to be analyzed.
+        Upload a URL to be analyzed.
 
         :param str url: the url to analyze
         :param str|None referer: the referer
@@ -1059,6 +1148,73 @@ class AnalysisClient(AbstractClient):
             ret_data.pop("report", None)
         return ret_data
 
+    def yield_completed_tasks(
+        self,
+        submissions: List[Dict],
+        start_timestamp: datetime.datetime,
+        wait_completion_interval_seconds: float = 15.0,
+        wait_completion_max_seconds: Optional[float] = None,
+        wait_max_num_tries: int = 5,
+    ) -> Generator[Dict, None, None]:
+        """
+        Returns a generator, which gives completed tasks as soon as they are ready.
+
+        :param list[dict] submissions: dictionary of submissions as returned by 'submit_*'
+        :param datetime.datetime start_timestamp: timestamp before the first submission happened
+        :param float wait_completion_max_seconds: do not wait for longer than this
+        :param float wait_completion_interval_seconds: how long to wait between polls
+        :param int wait_max_num_tries: maximum number of attempts
+        :rtype: generator[dict]
+        :return: generator that yields completed tasks
+        :raises WaitResultTimeout: when waiting for results timed out
+        """
+
+        def _get_timeout_or_raise() -> float:
+            end_completion_time = (
+                time.time() + wait_completion_max_seconds
+                if wait_completion_max_seconds is not None
+                else None
+            )
+            sleep_timeout = wait_completion_interval_seconds
+            if end_completion_time is not None:
+                now = time.time()
+                if now >= end_completion_time:
+                    raise exceptions.WaitResultTimeout()
+                if now + sleep_timeout > end_completion_time:
+                    sleep_timeout = end_completion_time - now
+            return sleep_timeout
+
+        attempts = 0
+        pending_submissions = {}
+        for submission in submissions:
+            if "score" in submission:
+                yield submission
+            else:
+                pending_submissions[submission["task_uuid"]] = submission
+        while pending_submissions:
+            try:
+                ret = self.get_completed(after=start_timestamp, include_score=True)
+            except exceptions.CommunicationError:
+                attempts += 1
+                if attempts > wait_max_num_tries:
+                    raise
+            else:
+                attempts = 0
+                start_timestamp = ret["before"]
+                for task_uuid, score in ret["tasks"].items():
+                    try:
+                        submission = pending_submissions[task_uuid]
+                    except KeyError:
+                        continue
+                    submission["score"] = score
+                    del pending_submissions[task_uuid]
+                    yield submission
+                if ret["more_results_available"]:
+                    continue
+                if not pending_submissions:
+                    break
+            time.sleep(_get_timeout_or_raise())
+
 
 class MultiClientMixin:
     """
